use palette::Oklch;

#[derive(Debug, Clone)]
pub struct NeutralSwatch(Oklch, Oklch);

impl NeutralSwatch {
    pub fn new(color1: Oklch, color2: Oklch) -> Self {
        if color1.l < color2.l {
            NeutralSwatch(
                Oklch {
                    l: color1.l * 0.4,
                    chroma: color1.chroma * 0.1,
                    ..color1
                },
                Oklch {
                    l: color2.l * 1.6,
                    chroma: color2.chroma * 0.1,
                    ..color2
                },
            )
        } else {
            NeutralSwatch(
                Oklch {
                    l: color2.l * 0.4,
                    chroma: color2.chroma * 0.1,
                    ..color2
                },
                Oklch {
                    l: color1.l * 1.6,
                    chroma: color1.chroma * 0.1,
                    ..color1
                },
            )
        }
    }

    pub fn get(&self, percent: f32) -> Oklch {
        let start_hue = self.0.hue.into_positive_degrees();
        let end_hue = self.1.hue.into_positive_degrees();

        let hue = if (start_hue - end_hue).abs() > 180.0 {
            if end_hue > start_hue {
                start_hue + (end_hue + 360.0 - start_hue) * percent
            } else {
                start_hue + (end_hue - start_hue + 360.0) * percent
            }
        } else {
            start_hue + (end_hue - start_hue) * percent
        }
        .rem_euclid(360.0);

        Oklch {
            l: percent,
            chroma: self.0.chroma + (self.1.chroma - self.0.chroma) * percent,
            hue: hue.into(),
        }
    }
}
